@@ -1,7 +1,7 @@
 package HTML::TreeBuilder::LibXML;
 use strict;
 use warnings;
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 use Carp ();
 use base 'HTML::TreeBuilder::LibXML::Node';
 use XML::LibXML;
@@ -63,6 +63,7 @@ sub parse_file {
 sub eof {
     my ($self, ) = @_;
     $self->{_content} = ' ' if defined $self->{_content} && $self->{_content} eq ''; # HACK
+    $self->{_implicit_html} = 1 unless $self->{_content} =~ /<html/i; # TODO find a better way to know that implicit <html> was inserted
     my $doc = $self->_parser->parse_html_string($self->{_content});
     $self->{node} = $self->_documentElement($doc);
 }
@@ -80,6 +81,26 @@ sub elementify {
     bless shift, 'HTML::TreeBuilder::LibXML::Node';
 }
 
+sub guts {
+    my ($self, $destructive) = @_;
+    
+    #warn "# me: ". $self->{node};
+        
+    my @out = $self->{_implicit_html} ? $self->{node}->findnodes('/html/body/*')
+                                      : $self->{node};
+
+    
+    #warn "# OUT: @out";
+    
+    return map { HTML::TreeBuilder::LibXML::Node->new($_) } @out if wantarray;    # one simple normal case.
+    return unless @out;
+    return HTML::TreeBuilder::LibXML::Node->new($out[0]) if @out == 1 and ref( $out[0] );
+    my $div = XML::LibXML::Element->new('div'); # TODO put the _implicit flag somewhere, to be compatible with HTML::TreeBuilders
+    $div->appendChild($_) for @out;
+    $div->setOwnerDocument(XML::LibXML->createDocument( "1.0", "UTF-8" ));
+    return HTML::TreeBuilder::LibXML::Node->new($div);    
+}
+
 sub replace_original {
     require HTML::TreeBuilder::XPath;
 
@@ -117,6 +138,8 @@ sub DESTROY {
 1;
 __END__
 
+=encoding utf-8
+
 =head1 NAME
 
 HTML::TreeBuilder::LibXML - HTML::TreeBuilder and XPath compatible interface with libxml
@@ -163,7 +186,7 @@ This is a benchmark result by tools/benchmark.pl
 
 =head1 AUTHOR
 
-Tokuhiro Matsuno E<lt>tokuhirom  slkjfd gmail.comE<gt>
+Tokuhiro Matsuno E<lt>tokuhirom@gmail.comE<gt>
 
 Tatsuhiko Miyagawa E<lt>miyagawa@cpan.orgE<gt>
 
